"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullMonitor = void 0;
const data_sources_1 = require("./gql/data-sources");
const type_defs_1 = require("./gql/type-defs");
const resolvers_1 = require("./gql/resolvers");
const ui_1 = require("./ui");
const metrics_collector_1 = require("./metrics-collector");
const queue_1 = require("./queue");
const constants_1 = require("./constants");
class BullMonitor {
    constructor(config) {
        this._queuesMap = new Map();
        this._readonlyQueuesMap = new Map();
        this.gqlBasePath = '/graphql';
        this.config = this._normalizeConfig(config);
        this._ui = new ui_1.UI();
        this._initQueues(this.config.queues);
        if (this.config.metrics) {
            this._initMetricsCollector();
        }
    }
    get queues() {
        return this._queues;
    }
    set queues(queues) {
        this._initQueues(queues);
        if (this._metricsCollector && this.config.metrics) {
            this._metricsCollector.queues = this._queues;
        }
    }
    createServer(Server) {
        this.server = new Server({
            typeDefs: type_defs_1.typeDefs,
            // @ts-ignore
            resolvers: resolvers_1.resolvers,
            playground: this.config.gqlPlayground,
            introspection: this.config.gqlIntrospection,
            dataSources: () => ({
                bull: new data_sources_1.BullDataSource(this._queues, this._queuesMap, {
                    textSearchScanCount: this.config.textSearchScanCount,
                }),
                metrics: new data_sources_1.MetricsDataSource(this._metricsCollector),
                policies: new data_sources_1.PoliciesDataSource(this._readonlyQueuesMap),
            }),
        });
    }
    startServer() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.server.start();
        });
    }
    renderUi() {
        return this._ui.render();
    }
    get baseUrl() {
        return this.config.baseUrl;
    }
    get uiEndpoint() {
        return this.baseUrl || '/';
    }
    get gqlEndpoint() {
        const base = this.baseUrl;
        if (!base) {
            return this.gqlBasePath;
        }
        else if (base.endsWith('/')) {
            return base.slice(0, -1) + this.gqlBasePath;
        }
        return base + this.gqlBasePath;
    }
    _initQueues(rawQueues) {
        const queues = rawQueues.map((queue) => Array.isArray(queue) ? queue[0] : queue);
        this._queues = queues.map(queue_1.patchBullQueue);
        this._queues.forEach((queue, idx) => {
            this._queuesMap.set(queue.id, queue);
            const rawQueue = rawQueues[idx];
            if (Array.isArray(rawQueue) && rawQueue[1].readonly) {
                this._readonlyQueuesMap.set(queue.id, queue);
            }
        });
    }
    _normalizeConfig(config) {
        return Object.assign(Object.assign(Object.assign({}, constants_1.DEFAULT_ROOT_CONFIG), config), { metrics: config.metrics
                ? Object.assign(Object.assign({}, constants_1.DEFAULT_METRICS_CONFIG), config.metrics) : false });
    }
    _initMetricsCollector() {
        this._metricsCollector = new metrics_collector_1.MetricsCollector(this._queues, this.config.metrics);
        this._metricsCollector.startCollecting();
    }
}
exports.BullMonitor = BullMonitor;
//# sourceMappingURL=main.js.map